
/**
***  Fichero:      CATFUN.C
***  Descripcion:  implementaciones del catalogo de funciones
**/

#include <stdlib.h>
#include <math.h>
#include <stdio.h>


#include "sbspline.h"
#include "catfun.h"


#ifndef M_PI
#define M_PI  3.141598
#endif

#define NPU 5  /** numero de picos en longitud   **/
#define NPV 3  /** numero de picos en latitud    **/
 

#define WU    (2.0*M_PI*(float)NPU)
#define WV    (2.0*M_PI*(float)NPV)
#define FV    (-0.5*M_PI)

real sqr(real x) { return x*x ; }

real radio( real u, real v )
{  real r= 0.25*(2.0+sin(u*WU)+sin(v*WV+FV)) ;
   return 1.0+0.5*sqr(sqr(sqr(sqr(r)))) ;
}

void plano_xz( punto *p, real u, real v )
{
   p->x = u ;
   p->z = v ;
   p->y = 0.0 ;
}

void esfera_picos( punto *p, real u, real v )
{
  real vv= M_PI*(v-0.5);
  real uu= M_PI*2.0*u  ;
  real r = radio(u,v) ;
  real cos_vv= cos(vv) ;

  asigna_punto( p, r*cos_vv*sin(uu), r*sin(vv), r*cos_vv*cos(uu) ) ;
}

void esfera( punto *p, real u, real v )
{
  real vv= M_PI*(v-0.5);
  real uu= M_PI*2.0*u  ;
  real cos_vv= cos(vv) ;

  asigna_punto( p, cos_vv*sin(uu), sin(vv), cos_vv*cos(uu) ) ;
}


void caracola( punto *p, real u, real v )
{
   float alfa, beta, rt, ra, rp, tmp ;

    /** este 'swap' es una chapuza .. pero funciona!
        (pone las normales y las orientaciones en el
         sentido 'correcto' segun los programas GRF y GIRT
     **/

   tmp = u ;
   u = v ;
   v = tmp ;


   alfa= v*6*M_PI ;
   beta= u*2*M_PI ;

   /** radio del tubo circular (ancho) 
   **/
   rt= 0.2+0.4*(1.0-v)*(1.0+0.1*cos(70*M_PI*v+16*M_PI*u))  ;


   /** radio del centro a la circunferencia interior del tubo 
   **/
   ra= 0.1+0.6*(1.0-v); 
       
   /** creacion del punto
   **/
   rp= ra+rt*cos(beta) ;
   asigna_punto( p, rp*cos(alfa), rt*sin(beta)+(1.5*v)-0.75, rp*sin(alfa)  ) ;
}

void caracola_simple( punto *p, real u, real v )
{
   float alfa, beta, rt, ra, rp, tmp ;

   /** este 'swap' es una chapuza .. pero funciona! **/

   tmp = u ;
   u = v ;
   v = tmp ;

   /** aqui va lo bueno: **/

   alfa= v*6*M_PI ;
   beta= u*2*M_PI ;

   /** radio del tubo circular (ancho)
   **/
   rt= 0.2+0.4*(1.0-v)  ;


   /** radio del centro a la circunferencia interior del tubo
   **/
   ra= 0.1+0.6*(1.0-v);

   /** creacion del punto
   **/
   rp= ra+rt*cos(beta) ;
   asigna_punto( p, rp*cos(alfa), rt*sin(beta)+(1.5*v)-0.75, rp*sin(alfa)  ) ;
}


void ondas( punto *p, real u, real v )
{
   double nx = 4.0 ;
   double ny = 4.0 ;

   p->x = u ;
   p->z = v ;

   p->y = 0.12*cos(nx*M_PI*u)*cos(ny*M_PI*v) ;
}

void hueco( punto *p, real u, real v )
{
   double nx = 4.0 ;
   double ny = 4.0 ;

   p->x = u ;
   p->z = v ;

   p->y = 0.1*cos(nx*M_PI*u)*cos(ny*M_PI*v) ;
}

void cilindro( punto *p, real u, real v )
{
   p->y = v ;

   p->x = cos(2.0*M_PI*u);
   p->z = sin(2.0*M_PI*u) ;
}

/** datos de los puntos de control que usara la funcion bspline
**/
sup_bspline * param_bspl= NULL ;

void parametros_bspline( sup_bspline *sbs )
{
   param_bspl= sbs ;
}

void bspline( punto *p, real u, real v )
{
  if ( param_bspl == NULL )
     error("bspline","debe usar primero parametros_bspline") ;

  eval_sup_bspline( param_bspl, p, u,v ) ;
}
