


#include "genmalla.h"
#include <stdio.h>
#include <math.h>

#define INDEX(iu,iv) (((iv)*(nu+1))+(iu))

double tx=0.0,ty=0.0,tz=0.0,e=1.0,   /** traslacion y escala **/
       crx=0.0, crz=0.0, angr =0.0 ; /** centro de rotacion, angulo de rotacion **/

double mr=1.0,mg=1.0,mb=1.0 ; /** color de las caras de la malla **/
int    color_malla=1 ; /** 0 --> no ponerle color a la malla, 1 --> si ponerle color **/


void transf( punto *p )
{
      double tmpx, tmpz ;

      tmpx = p->x - crx ;
      tmpz = p->z - crz ;

      p->x =  + tmpx*cos(angr) + tmpz*sin(angr) ;
      p->z =  - tmpx*sin(angr) + tmpz*cos(angr) ;

      p->x = p->x*e + tx + crx ;
      p->y = p->y*e + ty ;
      p->z = p->z*e + tz + crz ;
}

double dist( double u0, double v0, double u1, double v1 )
{
    return sqrt((u0-u1)*(u0-u1)+(v0-v1)*(v0-v1)) ;
}

void genmalla( fpar f, unsigned nu, unsigned nv )
{

   unsigned long i,iu, iv, ntotal ;

   double du = 1.0/(double)nu ,
          dv = 1.0/(double)nv ;

   double ir,ig,ib ;


   ntotal = (nu+1)*(nv+1) ;


   printf("\nbegin mesh") ;

   printf("\n   begin vertexs");

   for( i = 0 ; i < ntotal ; i++ )
   {

      real      u,v ;
      vector3   p,p01,p10 ;
      unsigned  iu,iv ;
      double    dx0,dy0,dz0,
                dx1,dy1,dz1,
                nx,ny,nz,
                mn ;

      iu = i % (nu+1) ;
      iv = i / (nu+1) ;

      u = (double)iu / (double)nu ;
      v = (double)iv / (double)nv ;

      f( &p, u, v ) ; transf(&p) ;


      f( &p01, u, v+dv ) ; transf(&p01) ;
      f( &p10, u+du, v ) ; transf(&p10) ;

      dx0 = p01.x - p.x ;
      dy0 = p01.y - p.y ;
      dz0 = p01.z - p.z ;

      dx1 = p10.x - p.x ;
      dy1 = p10.y - p.y ;
      dz1 = p10.z - p.z ;

      nx = dy0*dz1 - dy1*dz0 ;
      ny = dz0*dx1 - dz1*dx0 ;
      nz = dx0*dy1 - dx1*dy0 ;

      mn = sqrt( nx*nx + ny*ny + nz*nz ) ;

      //printf("\n      // u=%f v=%f",u,v) ;



      ir = 1.0-dist(u,v,0.33,0.33);
      ig = 1.0-dist(u,v,0.33,0.66);
      ib = 1.0-dist(u,v,0.66,0.50);

      ir = 0.5 + 0.5*sin(dist(u,v,0.50,0.50)*30.0) ;
      ig = 0.5 + 0.5*sin(dist(u,v,0.55,0.45)*30.0) ;
      ib = 0.5 + 0.5*sin(dist(u,v,0.45,0.55)*30.0) ;

      printf("\n      normal (%f,%f,%f)",nx/mn,ny/mn,nz/mn) ;
      printf("\n      irad (%f,%f,%f)",ir,ig,ib) ;
      printf("\n      uv (%f,%f)",u,v);
      printf("\n      (%f,%f,%f)",p.x,p.y,p.z) ;
   }

   printf("\n   end vertexs") ;

   printf("\n   begin faces") ;

   if ( color_malla )
       printf("\n      rgb (%f,%f,%f)",(float)mr,(float)mg,(float)mb);

   color_malla = 0 ;

   for( iu=0 ; iu < nu ; iu++ )
   for( iv=0 ; iv < nv ; iv++ )
   {
      unsigned
         i00 = INDEX(iu  ,iv  ) ,
         i01 = INDEX(iu  ,iv+1) ,
         i10 = INDEX(iu+1,iv  ) ,
         i11 = INDEX(iu+1,iv+1) ;

      printf("\n      (%d,%d,%d)",i00,i01,i10) ;
      printf("\n      (%d,%d,%d)",i10,i01,i11) ;

   }

   printf("\n   end faces") ;
   printf("\n ") ;
   printf("\nend mesh") ;
   printf("\n\n") ;

}

